@@ -1,8 +1,24 @@
+# ClamTk, copyright (C) 2004-2010 Dave M
+#
+# This file is part of ClamTk.
+#
+# ClamTk is free software; you can redistribute it and/or modify it
+# under the terms of either:
+#
+# a) the GNU General Public License as published by the Free Software
+# Foundation; either version 1, or (at your option) any later version, or
+#
+# b) the "Artistic License".
 package ClamTk::Results;
 
 use strict;
+#use warnings;    # disabled upon release
+$|++;
+
 use File::Basename;
 use File::Copy;
+use Digest::MD5 qw/md5_hex/;
+use Carp qw/cluck/;
 
 use encoding 'utf8';
 
@@ -10,21 +26,18 @@ use Gtk2;
 use Gtk2::SimpleList;
 use Glib qw/TRUE FALSE/;
 
-use ClamTk::GUI;
-
 use Locale::gettext;
 use POSIX qw/locale_h/;
-textdomain("clamtk");
-setlocale( LC_MESSAGES, "" );
-bind_textdomain_codeset( "clamtk", "UTF-8" );
+textdomain('clamtk');
+setlocale( LC_MESSAGES, '' );
+bind_textdomain_codeset( 'clamtk', 'UTF-8' );
 
 my $popup;
 my $slist;
 my $hover_label;
 
 sub display {
-    my ($class)    = shift;
-    my ($self)     = shift;
+    shift;    # Don't need package name
     my ($cmd_scan) = shift;
     my ($hash)     = @_;
 
@@ -84,7 +97,7 @@ sub display {
 
     my $tooltips = Gtk2::Tooltips->new;
     $tooltips->set_tip( $slist,
-        gettext("Select a file and right-click for options...") );
+        gettext('Select a file and right-click for options...') );
     $tooltips->disable;
 
     # below: the right-click functionality. also uses 'sub confirm'.
@@ -96,60 +109,70 @@ sub display {
             my $deref = $sel[0];
             defined $deref or return;
 
+            # We have to avoid parsing mail for now
+            my $maildirs = join( '|',
+                '.thunderbird', '.mozilla-thunderbird',
+                '.evolution',   'Mail',
+                'kmail' );
+
             my $menu = Gtk2::Menu->new();
+
+            # Right-click 'quarantine' option
             my $quar_pop =
                 Gtk2::ImageMenuItem->new( gettext('Quarantine this file') );
             my $quar_image =
                 Gtk2::Image->new_from_stock( 'gtk-refresh', 'menu' );
             $quar_pop->set_image($quar_image);
-            if ( $hash->{$deref}->{name}
-                =~ /\.(thunderbird|mozilla-thunderbird|evolution|mozilla)/ )
-            {
+            if ( $hash->{$deref}->{name} =~ /$maildirs/ ) {
                 $hover_label->set_text(
                     gettext(
                         'Possible email file - please manually quarantine or delete it.'
                     )
                 );
-            } elsif ( -d $hash->{$deref}->{name} ) {
+            }
+            elsif ( -d $hash->{$deref}->{name} ) {
                 return;
-            } else {
+            }
+            else {
                 $quar_pop->signal_connect(
-                    activate => sub { main_confirm( $deref, "q", \$hash ) } );
+                    activate => sub { main_confirm( $deref, 'q' ) } );
             }
             $quar_pop->show();
             $menu->append($quar_pop)
                 unless dirname( $hash->{$deref}->{name} )
                     =~ /^\/(proc|sys|dev)/;
 
+            # Right-click 'delete' option
             my $delete_pop =
                 Gtk2::ImageMenuItem->new( gettext('Delete this file') );
             my $del_image =
                 Gtk2::Image->new_from_stock( 'gtk-delete', 'menu' );
             $delete_pop->set_image($del_image);
-            if ( $hash->{$deref}->{name}
-                =~ /\.(thunderbird|mozilla-thunderbird|evolution|mozilla)/ )
-            {
+            if ( $hash->{$deref}->{name} =~ /$maildirs/ ) {
                 $hover_label->set_text(
                     gettext(
                         'Possible email file - please manually quarantine or delete it.'
                     )
                 );
-            } elsif ( -d $hash->{$deref}->{name} ) {
+            }
+            elsif ( -d $hash->{$deref}->{name} ) {
                 return;
-            } else {
+            }
+            else {
                 $delete_pop->signal_connect(
-                    activate => sub { main_confirm( $deref, "d", \$hash ) } );
+                    activate => sub { main_confirm( $deref, 'd' ) } );
             }
             $delete_pop->show();
             $menu->append($delete_pop)
                 unless dirname( $hash->{$deref}->{name} )
                     =~ /^\/(proc|sys|dev)/;
 
+            # Right-click 'save-as' option
             my $save_pop =
                 Gtk2::ImageMenuItem->new_from_stock( 'gtk-save-as', undef );
             $save_pop->signal_connect(
                 activate => sub {
-                    return if ( -d $hash->{$deref}->{name} );
+                    return if ( -d $slist->{data}[$deref][0] );
                     my $save_dialog = Gtk2::FileChooserDialog->new(
                         gettext('Save As...'), undef, 'save',
                         'gtk-cancel' => 'cancel',
@@ -161,18 +184,19 @@ sub display {
                     if ( "ok" eq $save_dialog->run ) {
                         my $tmp = $save_dialog->get_filename;
                         $save_dialog->destroy();
-                        move( $hash->{$deref}->{name}, $tmp )
+                        move( $slist->{data}[$deref][0], $tmp )
                             or do {
                             show_message_dialog( $popup, 'error', 'close',
-                                gettext("Could not save that file.") );
+                                gettext('Could not save that file.') );
                             return FALSE;
                             };
-                        $hash->{$deref}->{action} = gettext('Moved');
-                        main_confirm( $deref, "s", \$hash );
+                        $slist->{data}[$deref][2] = gettext('Moved');
+                        main_confirm( $deref, 's' );
                         show_message_dialog( $popup, 'info', 'close',
-                            gettext("File saved.") );
+                            gettext('File saved.') );
                         return FALSE;
-                    } else {
+                    }
+                    else {
                         $save_dialog->destroy();
                     }
                 }
@@ -180,6 +204,7 @@ sub display {
             $save_pop->show();
             $menu->append($save_pop);
 
+            # Right-click 'cancel' option
             my $cancel_pop =
                 Gtk2::ImageMenuItem->new_from_stock( 'gtk-cancel', undef );
             $cancel_pop->signal_connect( activate => sub { return; } );
@@ -198,113 +223,137 @@ sub display {
 sub main_confirm {
     my $number  = shift;
     my $do_this = shift;
-    my ($found) = shift;
 
+    my $full_name = $slist->{data}[$number][0];
+    my $status    = $slist->{data}[$number][1];
+    my $action    = $slist->{data}[$number][2];
+
+    # do_this = quarantine (q), delete (d), or save (s)
     if ( $do_this eq "q" ) {
-        if ( not -e $$found->{$number}->{name} ) {
+        if ( not -e $full_name ) {
             $hover_label->set_text(
-                sprintf gettext("File has been moved or deleted already.") );
+                sprintf gettext('File has been moved or deleted already.') );
             $slist->{data}[$number][0] =
-                "<span foreground='#CCCCCC'>$$found->{$number}->{name}</span>";
+                "<span foreground='#CCCCCC'>$full_name</span>";
             $slist->{data}[$number][1] =
-                "<span foreground='#CCCCCC'>$$found->{$number}->{status}</span>";
+                "<span foreground='#CCCCCC'>$status</span>";
             $slist->{data}[$number][2] =
-                "<span foreground='#CCCCCC'>$$found->{$number}->{action}</span>";
+                "<span foreground='#CCCCCC'>$action</span>";
             return;
         }
-        if ( move_to_quarantine( $number, $found ) ) {
-            $hover_label->set_text( gettext("File has been quarantined.") );
-            $$found->{$number}->{action} =
+        if ( move_to_quarantine($number) ) {
+            $hover_label->set_text( gettext('File has been quarantined.') );
+            $slist->{data}[$number][2] =
                 sprintf gettext("<b>Quarantined</b>");
-        } else {
+        }
+        else {
             $hover_label->set_text(
-                gettext("File could not be quarantined.") );
+                gettext('File could not be quarantined.') );
             return;
         }
-    } elsif ( $do_this eq "d" ) {
-        if ( not -e $$found->{$number}->{name} ) {
+    }
+    elsif ( $do_this eq "d" ) {
+        if ( not -e $full_name ) {
             $hover_label->set_text(
-                gettext("File has been moved or deleted already.") );
+                gettext('File has been moved or deleted already.') );
             $slist->{data}[$number][0] =
-                "<span foreground='#CCCCCC'>$$found->{$number}->{name}</span>";
+                "<span foreground='#CCCCCC'>$full_name</span>";
             $slist->{data}[$number][1] =
-                "<span foreground='#CCCCCC'>$$found->{$number}->{status}</span>";
+                "<span foreground='#CCCCCC'>$status</span>";
             $slist->{data}[$number][2] =
-                "<span foreground='#CCCCCC'>$$found->{$number}->{action}</span>";
+                "<span foreground='#CCCCCC'>$action</span>";
             return;
         }
-        my $confirm_message = gettext("Really delete this file?");
+        my $confirm_message = gettext('Really delete this file?');
         my $confirm =
             Gtk2::MessageDialog->new( $popup, [qw(modal destroy-with-parent)],
             'question', 'ok-cancel', $confirm_message );
 
-        if ( "cancel" eq $confirm->run ) {
+        if ( 'cancel' eq $confirm->run ) {
             $confirm->destroy;
             return;
-        } else {
+        }
+        else {
             $confirm->destroy;
-            if ( unlink( $$found->{$number}->{name} ) ) {
-                $hover_label->set_text( gettext("File has been deleted.") );
-                $$found->{$number}->{action} =
+            if ( unlink($full_name) ) {
+                $hover_label->set_text( gettext('File has been deleted.') );
+                $slist->{data}[$number][2] =
                     sprintf gettext("<b>Deleted</b>");
-            } else {
+            }
+            else {
                 $hover_label->set_text(
-                    gettext("File could not be deleted.") );
+                    gettext('File could not be deleted.') );
                 return;
             }
         }
-    } elsif ( $do_this eq 's' ) {
-
-        #if ( not -e $$found->{$number}->{name} ) {
-        #$hover_label->set_text(
-        #    sprintf gettext("File has been moved or deleted already.") );
+    }
+    elsif ( $do_this eq 's' ) {
+        my $new_action = gettext('Moved');
         $slist->{data}[$number][0] =
-            "<span foreground='#CCCCCC'>$$found->{$number}->{name}</span>";
+            "<span foreground='#CCCCCC'>$full_name</span>";
         $slist->{data}[$number][1] =
-            "<span foreground='#CCCCCC'>$$found->{$number}->{status}</span>";
+            "<span foreground='#CCCCCC'>$status</span>";
         $slist->{data}[$number][2] =
-            "<span foreground='#CCCCCC'>$$found->{$number}->{action}</span>";
+            "<span foreground='#CCCCCC'><b>$new_action</b></span>";
         return;
     }
-    main_slist_delete( $number, $found );
+    main_slist_delete($number);
+    return;
 }
 
 sub main_slist_delete {
-    my $number = shift;
-    my ($found) = shift;
+    my $number    = shift;
+    my $full_name = $slist->{data}[$number][0];
+    my $status    = $slist->{data}[$number][1];
+    my $action    = $slist->{data}[$number][2];
     $slist->{data}[$number][0] =
-        "<span foreground='#CCCCCC'>$$found->{$number}->{name}</span>";
-    $slist->{data}[$number][1] =
-        "<span foreground='#CCCCCC'>$$found->{$number}->{status}</span>";
-    $slist->{data}[$number][2] =
-        "<span foreground='#CCCCCC'>$$found->{$number}->{action}</span>";
+        "<span foreground='#CCCCCC'>$full_name</span>";
+    $slist->{data}[$number][1] = "<span foreground='#CCCCCC'>$status</span>";
+    $slist->{data}[$number][2] = "<span foreground='#CCCCCC'>$action</span>";
     $popup->queue_draw;
+    return;
 }
 
 sub move_to_quarantine {
-    my $number   = shift;
-    my ($found)  = shift;
-    my $basename = basename( $$found->{$number}->{name} );
-    if (   not -e $main::scanner->{viruses}
-        or not -d $main::scanner->{viruses} )
+    my $number = shift;
+
+    my $full_name = $slist->{data}[$number][0];
+    my $basename  = basename($full_name);
+
+    my $paths = ClamTk::App->get_path('viruses');
+
+    if (   not -e $paths
+        or not -d $paths )
     {
         show_message_dialog( $popup, 'info', 'close',
-            gettext("Quarantine directory does not exist.") );
+            gettext('Quarantine directory does not exist.') );
         return;
     }
-    chmod oct(600), $$found->{$number}->{name};
-    system(
-        "mv",
-        $$found->{$number}->{name},
-        "$main::scanner->{viruses}/$basename"
-    );
-    rename(
-        "$main::scanner->{viruses}/$basename",
-        "$main::scanner->{viruses}/$basename.VIRUS"
-    );
-    if ( not -e $$found->{$number}->{name} ) {
+
+    # Get permissions
+    my $mode = ( stat($full_name) )[2];
+    my $perm = sprintf( "%03o", $mode & oct(7777) );
+
+    chmod oct(600), $full_name;
+    system( 'mv', $full_name, "$paths/$basename" );
+    rename( "$paths/$basename", "$paths/$basename.VIRUS" );
+
+    # Update restore file by adding file, path and md5
+    my $ctx = Digest::MD5->new;
+    open( my $F, '<', "$paths/$basename.VIRUS" ) or do {
+        cluck "Could not open file for md5-ing: $!\n";
+        return;
+    };
+    $ctx->addfile(*$F);
+    my $md5 = $ctx->hexdigest;
+    close($F);
+
+    ClamTk::Prefs->restore( $md5, 'add', $full_name, $perm );
+
+    if ( not -e $full_name ) {
         return 1;
-    } else {
+    }
+    else {
         return -1;
     }
 }
